<?php
class Cryptophoto_CryptophotoAdmin_Plugin extends Atmail_Controller_Plugin
{
  protected $_pluginFullName = 'Cryptophoto Administrator Settings';
  protected $_pluginAuthor = 'Cryptophoto <Tech@CryptoPhoto.com>';
  protected $_pluginDescription = 'Improves security by adding the Cryptophoto two-factor authentication';
  protected $_pluginCopyright = '2012-2015 Cryptophoto';
  protected $_pluginUrl = '';
  protected $_pluginNotes = '';
  protected $_pluginVersion = '1.20150610';
  protected $_pluginCompat = '6.1.5';
  protected $_pluginModule = 'admin';
  private $_loginPage = false;

  // class constructor
  public function __construct()
  {
    $this->_pluginDescription = "Admin Management for Cryptophoto";
    $this->log = Zend_Registry::get('log');

    if (file_exists(dirname(__FILE__) . "/config.php")) {
      require_once(dirname(__FILE__) . "/config.php");

      $this->admin_publickey  = $_CP_ADMIN_CONFIG["admin_publickey"];
      $this->admin_privatekey = $_CP_ADMIN_CONFIG["admin_privatekey"];
      $this->admin_salt       = $_CP_ADMIN_CONFIG["admin_salt"];
      $this->admin_server     = $_CP_ADMIN_CONFIG["admin_server"];
    }   
    
    parent::__construct();
  }

  /*
  * creates the Cryptophoto tables upon installation
  */
  public function setup()
  {
    $db = Zend_Registry::get("dbAdapter");
    if(!$this->dbValid('#__cryptophotokeys')) {
      $db->query("CREATE TABLE `#__cryptophotokeys` (`rowid` int(11) unsigned NOT NULL auto_increment, `publickey` varchar(120) NOT NULL DEFAULT '', `privatekey` varchar(120) NOT NULL DEFAULT '', `salt` varchar(120) NOT NULL DEFAULT '', `server` varchar(120) NOT NULL DEFAULT '',PRIMARY KEY  (`rowid`))");
      $db->query("INSERT INTO `#__cryptophotokeys` (`publickey`,`privatekey`,`salt`,`server`) VALUES('','','','https://cryptophoto.com')");
    }
    if(!$this->dbValid('#__cryptophoto')) {
      $db->query("CREATE TABLE `#__cryptophoto` (`rowid` int(11) unsigned NOT NULL auto_increment, `account` varchar(128) NOT NULL DEFAULT '', `enabledCP` varchar(11) NOT NULL DEFAULT '',`mailserver` varchar(64) NOT NULL DEFAULT '',`password` varchar(128) NOT NULL DEFAULT '',`expire` varchar(15) DEFAULT NULL, PRIMARY KEY  (`rowid`))");
    }
  }

  public function _renderSecurityTabNav()
  {
    $this->log->debug("**\n Admin: renderSecurityTabNav \n**\n");

    $params = array(
      "title" => "CryptoPhoto Protection",
      "text"  => "CryptoPhoto Protection",
      "icon"  => "images/cryptophoto.png"
    );
    
    $class = get_class($this);
    $id = strtolower($class) . "_adminsettings";
    $style = '';
    
    if (isset($params['icon'])) {
      $path = $this->_makeUrl($params['icon']);
      $style = "background: url($path) no-repeat 15px;display:block;height:19px;overflow:hidden;padding:11px 8px 6px 37px;white-space:nowrap;font-weight:bold";
    }
    
    $baseUrl = $this->_getModuleBaseUrl();
    
    echo "<li id=\"$id\"><a href=\"$baseUrl/plugininterface/index/$class/adminsettings\" title=\"{$params['title']}\"><span style=\"$style\" class\"label\">{$params['text']}</span></a></li>";
  }

  public function adminsettings()
  {
    $this->_initView();
    $userData = Zend_Auth::getInstance()->getStorage()->read();

    $this->log->debug("#".print_r($userData, true)."#");
    // get the user's id
    $this->userid = $userData;

    // properly installed Cryptophoto
    if($this->dbValid("#__cryptophoto") && $this->userid) {
      $db = Zend_Registry::get("dbAdapter");

      require_once(dirname(__FILE__) . "/CryptoPhotoUtils.php");
      
      // start a new Cryptophoto session
      $cp = new CryptoPhotoUtils($this->admin_privatekey, $this->admin_publickey, md5($this->admin_salt . $this->userid), $this->admin_server);
      $answer = $cp->start_session($_SERVER['REMOTE_ADDR'], array('interstitial_page_ops'=>1|2));

      if($answer[0] == TRUE) {
        $this->view->gen_widget = $cp->get_gen_widget();
        $this->view->image_url = $this->_makeUrl("images/loading.gif");
        $db = Zend_Registry::get("dbAdapter");
        $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
        $this->view->enabledCP = 0;
        // user was found
        if(count($db_values)) {
          $enabledCP = $db_values[0]["enabledCP"];
          if($enabledCP) {
            $this->view->enabledCP = 1;
          }
        }
        else {
            // insert the user's details into the Cryptophoto table
            $db->query('INSERT INTO `#__cryptophoto` (`account`,`enabledCP`) VALUES(?,0)', array($this->userid));
        }
      }
      else {
        $this->view->error = "Cryptophoto is not properly configured yet. Please check your settings.";
        $this->log->debug("Cryptophoto Server has responded with the following error message: " . $answer[1]);
      }
    }
    // cryptophoto table was not found - was deleted manually or installation didn't work well
    else {
        $this->view->error = "Cryptophoto is not properly configured yet. Please check your settings.";
        $error = 1;
    }
    echo $this->view->render("adminsettings.phtml");
  }

  /*
  * Save a certain user's Cryptophoto settings, when changed
  */
  public function saveAdminSettings()
  {
    $this->_initView();
    $userData = Zend_Auth::getInstance()->getStorage()->read();

    $this->log->debug("USERDATA=\n" . print_r($userData, true));
    //get the user's id
    $this->userid = $userData;

    $request = $this->getRequest()->getParams();

    $this->log->debug("REQUEST=\n" . print_r($request, true));

    $this->log->debug("userid = ". $this->userid);
    require_once("library/jQuery/jQuery.php");
    
    // check for valid CP database
    if($this->dbValid("#__cryptophoto")) {
    
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ? ", $this->userid);
    
      // the user exists in the CP database
      if(count($db_values)) {
        // valid requests for saving new settings
        if (!empty($request['save'])) {
          // enabled user's Cryptophoto
          if (!empty($request['enableCpCheckbox'])) {
            $db->query('UPDATE `#__cryptophoto` SET enabledCP = 1 WHERE account = ?', array($this->userid));
          }
          // disable user's Cryptophoto
          else {
            $db->query('UPDATE `#__cryptophoto` SET enabledCP = 0 WHERE account = ?', array($this->userid));
          }
          // success
          jQuery::addMessage("The settings have been updated");
        }
        // request was made with empty 'save' parameter
        else {
          $this->view->error = "Invalid request";
        }
      }
      // user not found in CP table though it should be - corrupted installation or files/db records were removed manually
      else {
        $this->view->error = "Something went wrong. This user does not exist in the Cryptophoto database";
      }
    }
    // cryptophoto table not found - admin plugin not installed or incmplete/corrupted installation
    else {
      $this->view->error = "The settings could not be updated. Cryptophoto is not properly installed or configured";
    }

    // render the CP widget
    echo $this->view->render("jsonresponse.phtml", null, true);
  }

  /*
  * Method for capturing a successful login event
  */
  public function loginAdminSuccess()
  {
    $request = $this->getRequest()->getParams();

    $this->userid = $request['Username'];

    if($this->dbValid("#__cryptophoto")) {

      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));

      // user exists in the Crpytophoto database
      if(count($db_values)) {

        $settings = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
        $salt = $settings[0]['salt'] ? $settings[0]['salt'] : "";

        // get the user's CP status

        if (isset($request['cpn']) && $request['cpt'] && $request['cpn'] == md5($this->userid . $request['Password'] . $this->admin_salt . $request['cpt'])) {
          return;
        }

        $enabledCP = $db_values[0]["enabledCP"];

        // Cryptophoto is enabled
        if($enabledCP) {
          $auth = Zend_Auth::getInstance();
          $auth->clearIdentity();
          // save the user's password, to be able to differentiate between a guest or a logged-in user when displaying the CP authentication page
          $db->query("UPDATE `#__cryptophoto` SET password=?, expire=? WHERE account = ?", array($request['Password'], time() + 10*60, $this->userid));
          //redirect the user to the Cryptophoto authentication page
          $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
          $class = get_class($this);
          $r->gotoUrl("/admin/plugininterface/index/$class/cpauth/id/" . $this->userid);
          return;

        }
      }
      // user not found in the Cryptophoto table
      else {
        $this->view->error = "Something went wrong with your Cryptophoto settings";
      }
    }
    // Cryptophoto table not found - invalid or corrupted installation
    else {
      $this->view->error = "Cryptophoto is not properly installed or configured";
    }
  }

  /*
   * Cryptophoto authentication
  */
  public function cpauth() {

$this->log->debug("*** in cpauth");

    require_once(dirname(__FILE__) . "/CryptoPhotoUtils.php");

    $this->_initView();
    $request = $this->getRequest()->getParams();
    $this->log->debug("REQUEST=\n" . print_r($request, true));
    
    // get the current user's id
    $this->userid = $request['id'];
    $db = Zend_Registry::get("dbAdapter");

    $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
    
    // get the user's password - the password is saved if the user arrives to the CP auth page after a successful login
    if (count($db_values)) {
      
      // if empty - redirect to the original login page - it means the user reach the page through URL
      if (!$db_values[0]["password"]) {
        $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
        $r->gotoUrl('/');
        return;
      } 
      else {
        $expire = $db_values[0]["expire"];
        if (time() > $expire) {
          $db->query("UPDATE `#__cryptophoto` SET password='', mailserver='', expire='0' WHERE account = ?", array($this->userid));
          $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
          $r->gotoUrl('/');
          return;
        } 
        else {
          $db->query("UPDATE `#__cryptophoto` SET expire=? WHERE account = ?", array(time() + 10 * 60, $this->userid));
        }
      }
    }
    
    // user not in Cryptophoto table or missing Cryptophoto table - deleted or bad installation)  - normally, this should never be reached
    else {
      $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
      $r->gotoUrl('/');
      return;
    }
    
    // valid user
    if ($this->userid) {

      $error = 0;
      
      // check for attempts to post empty values - don't even post to CP  if empty
      if (!empty($request['authenticate']) || !empty($request['authenticate'])) {
        
        // instantiate a new Cryptophoto object
        $authcp = new CryptoPhotoUtils($this->admin_privatekey, $this->admin_publickey, md5($this->admin_salt . $this->userid), $this->admin_server);
        
        // check for non-empty row and column codes
        if ((!empty($request["token_response_field_row"]) && !empty($request["token_response_field_col"])) || (!empty($request["cp_phc"]))) {
          
          $selector = $request["token_selector"];
          $row_code = $request["token_response_field_row"] ? $request["token_response_field_row"] : "";
          $col_code = $request["token_response_field_col"] ? $request["token_response_field_col"] : "";
          $cp_phc = $request["cp_phc"] ? $_POST["cp_phc"] : "";
          $ip = $_SERVER['REMOTE_ADDR'];
          
          // send codes to CP for authentication
          $authcp->verify_response($selector, $row_code, $col_code, $cp_phc, $ip);
          
          // codes ok. let's finally login this user
          if ($authcp->is_valid) {
            if ($this->dbValid("#__cryptophoto")) {
              $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
              
              // user found
              if (count($db_values)) {
                
                // save the user's password,
                $db->query("UPDATE `#__cryptophoto` SET password='', mailserver='' WHERE account = ?", array($this->userid));

                $this->view->Username = $this->userid;
                $this->view->Password = $db_values[0]['password'];
                $this->view->time = time();
                $this->view->nonce = md5($this->userid . $db_values[0]['password'] . $this->admin_salt . $this->view->time);
              }
              
              // user not found in the Cryptophoto table
              else {
                $this->view->error = "Something went wrong with your Cryptophoto settings";
                $error = 1;
              }
              
              return $this->view->render("autopost.phtml");
            }
          }
          
          // Cryptophoto authenticaion failed
          else {
            $this->view->error = "Invalid codes. Try again!";
            $error = 1;
          }
        }
        
        // provided codes are empty
        else {
          $this->view->error = "The codes may not be empty!";
        }
      }
      
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` WHERE account = ?", array($this->userid));
   
      if (count($db_values)) {
        
        // new Cryptophoto object and sessoin
        $cp = new CryptoPhotoUtils($this->admin_privatekey, $this->admin_publickey, md5($this->admin_salt . $this->userid), $this->admin_server);
        $answer = $cp->start_session($_SERVER['REMOTE_ADDR'],array('authentication'=>'true'));

        if ($answer[0] == TRUE) {
          
          // get the user's CP widget
          $this->view->gen_auth = $cp->get_auth_widget();
          $this->view->image_url = $this->_makeUrl("images/loading.gif");
        } 
        else {
          $this->view->error = "Something went terribly wrong with your Cryptophoto settings";
          $this->log->debug("Cryptophoto Server has responded with the following error message: " . $answer[1]);
        }
      }
      
      // user not found in CP table or the table itself is missing
      else {
        $this->view->error = "Something went terribly wrong with your Cryptophoto settings";
        $error = 1;
      }
      
      return $this->view->render("cpauth.phtml");
    }
    
    // user not found or CP table missing
    else {
      $r = Zend_Controller_Action_HelperBroker::getStaticHelper('redirector');
      $r->gotoUrl('/');
      return;
    }
  }


  /*
  * adds the Cryptophoto menu item in the settings section of the admin account
  */
  public function renderSettingsTabNav()
  {
    $this->log->debug("**\n Admin: renderMailSettingsNav \n**\n");
    $params = array(
      "title" => "CryptoPhoto Settings",
      "text"  => "CryptoPhoto",
      "icon"  => "images/cryptophoto.png"
    );
    echo $this->_createMailSettingsNavSection($params);
  }

  /*
  * The Cryptophoto settings section of the admin account
  */
  public function settings()
  {
    $this->_initView();
    $this->log->debug("**\n Admin: settings \n**\n");
    $db = Zend_Registry::get("dbAdapter");

    if($this->dbValid('#__cryptophotokeys')) {

      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
      $settings = $db_values[0];

      //check for existing Cryptophoto keys
      if($settings != null) {
        $this->view->cpvalid = true;
        $this->view->cp_publickey  = $settings['publickey'] ? $settings['publickey'] : "";
        $this->view->cp_privatekey = $settings['privatekey'] ? $settings['privatekey'] : "";
        $this->view->cp_salt       = $settings['salt'] ? $settings['salt'] : "";
        $this->view->cp_server     = $settings['server'] ? $settings['server'] : "https://cryptophoto.com";
      }
      // Cryptophoto keys not found or missing table
      else {
        $this->view->value = false;
        $this->view->err_message = "Could not detect a valid Cryptophoto installation";
        return;
      }
    }
    else {
      $this->view->value = false;
      $this->view->err_message = "Could not detect a valid Cryptophoto setup - Cryptophoto keys missing or invalid installation";
      return;
    }
    // render the admin settings page
    echo $this->view->render("settings.phtml");
  }
  /*
  * saves the site's Cryptophoto configuration when changing the keys and salt
  */
  public function saveSettings($settings=array())
  {
    $this->_initView();
    require_once("library/jQuery/jQuery.php");

    if($this->dbValid('#__cryptophotokeys')) {
      // fetch the current Cryptophoto settings
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
      $settings = $db_values[0];
      $publickey  = $settings['publickey'] ? $settings['publickey'] : "";
      $privatekey = $settings['privatekey'] ? $settings['privatekey'] : "";
      $salt       = $settings['salt'] ? $settings['salt'] : "";
      $server     = $settings['server'] ? $settings['server'] : "https://cryptophoto.com";
      $rowid      = $settings['rowid'];

      // non-empty salt
      if (!empty($_POST['cp_salt'])) {
        $salt  = $_POST['cp_salt'];
      }
      
      // non-empty privatekey
      if (!empty($_POST['cp_privatekey'])) {
        $privatekey  = $_POST['cp_privatekey'];
      }
      
      // non-empty publickey
      if (!empty($_POST['cp_publickey'])) {
        $publickey  = $_POST['cp_publickey'];
      }

      // non-empty server
      if (!empty($_POST['cp_server'])) {
        $server = $_POST['cp_server'];
      }
      
      $db->query("UPDATE `#__cryptophotokeys` SET `publickey`=?, `privatekey`=?, `salt`=?, `server`=? WHERE `rowid`=?", array(trim($publickey),trim($privatekey),$salt,$server,$rowid));
      jQuery::addMessage("The settings have been updated");
    }
    else {
      jQuery::addMessage("Could not detect a valid Cryptophoto installation");
      return;
    }

    echo $this->view->render("jsonresponse.phtml", null, true);
  }

  /*
  * Tests the configuration of the plugin
  */
  public function testSetup()
  {
    $this->_initView();
    require_once("library/jQuery/jQuery.php");
    if($this->dbValid('#__cryptophotokeys')) {
      // fetch the current Cryptophoto settings
      $db = Zend_Registry::get("dbAdapter");
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophotokeys`");
      $settings = $db_values[0];
      $publickey  = $settings['publickey'] ? $settings['publickey'] : "";
      $privatekey = $settings['privatekey'] ? $settings['privatekey'] : "";
      $salt       = $settings['salt'] ? $settings['salt'] : "";
      $server     = $settings['server'] ? $settings['server'] : "https://cryptophoto.com";

      if($publickey && $privatekey && $salt) {
      
        require_once(dirname(__FILE__) . "/CryptoPhotoUtils.php");
      
        $cp = new CryptoPhotoUtils($privatekey, $publickey, md5($salt), $server);
        $answer = $cp->start_session($_SERVER['REMOTE_ADDR']);
      
        if($answer[0] == TRUE) {
          jQuery::addMessage("Plugin is setup properly");
        } else {
          if ($answer[1] == "incorrect-timestamp") {
            jQuery::addError('Error: Please verify that the date/time is properly set on this server');
          }
          else if (strpos($answer[1], "ip-not-allowed") !== false) {
            jQuery::addError('Error: The IP of this server(' . $answer[2] . ') is not added to the list of allowed IPs. Check your settings in your CryptoPhoto.com account');
          }
          else {
            jQuery::addError('Error: Please verify that the API keys are valid');
          }
        }
      } else {
        jQuery::addError('Error: Please configure/save your API Keys and salt before testing the setup');
      }
    } else {
      jQuery::addMessage("Could not detect a valid Cryptophoto installation");
      return;
    }
    echo $this->view->render("jsonresponse.phtml", null, true);
  }

  /*
  * Displays the users list and their Cryptophoto status
  */
  public function userslist()
  {
    $this->_initView();
    $request = $this->getRequest()->getParams();
    $db = Zend_Registry::get("dbAdapter");
    if($this->dbValid('#__cryptophoto')) {

      // check for a valid post (non-empty CP status and user id)
      if(!empty($request['enable']) && !empty($request['id'])) {
      
        // enable Cryptophoto for a certain user
        $sql = "UPDATE `#__cryptophoto` SET enabledCP = 1 WHERE ";
        $where = str_repeat ( "rowid = ? OR ", count($request['id']) );
        $where = preg_replace('/OR $/', '', $where);
        $sql = $sql . $where;
        $db->query($sql, $request['id']);
      
      } else if (!empty($request['disable']) && !empty($request['id'])) {
        // disable Cryptophoto for a certain user
        $sql = "UPDATE `#__cryptophoto` SET enabledCP = 0 WHERE ";
        $where = str_repeat ( "rowid = ? OR ", count($request['id']) );
        $where = preg_replace('/OR $/', '', $where);
        $sql = $sql . $where;
        $db->query($sql, $request['id']);
      }
      
      // search filter by username
      $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto`");
      $filter = !empty($request['filter']) ? $request['filter'] : '';
      
      // check for posted filter
      if ($filter) {
        $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto` where account like ?", array("%".$filter."%"));
      } else {
        $db_values = $db->fetchAll("SELECT * FROM `#__cryptophoto`");
      }
      
      // check for posted page number; default page number = 1
      $page = !empty($request['page']) ? $request['page'] : 1;
      $paginator = Zend_Paginator::factory($db_values);
      
      // pagination settings
      $paginator->setItemCountPerPage(10);
      $paginator->setCurrentPageNumber($page);
      $this->view->userslist = $paginator;
    }
    else {
      jQuery::addMessage("Could not detect a valid Cryptophoto installation");
    }
    
    if( !isset($this->view->jsonIdsToRender) )
      $this->view->jsonIdsToRender = array();
    
    $this->view->jsonIdsToRender['users-table'] = 'listusers.phtml';
    
    echo $this->view->render('jsonresponse.phtml', null, true);
  }

  /*
   * authRequired
   */
  public function authRequired($method) {
    if ($method == "cpauth") {
      return false;
    }
    return true;
  }
  /*
  * Checks that the Cryptophoto table exists
  */
  public function dbValid($tableName) {

    $db = Zend_Registry::get("dbAdapter");
    $db_values = $db->fetchAll("SHOW TABLES LIKE '" . $tableName . "'");

    if(count($db_values)) {
      return true;
    }
    else {
      return false;
    }
  }
}